/*
 * History
 *
 * 09.11.2011 - [RH] - creation
 */
package com.sibvisions.module.user.screens;

import java.sql.Timestamp;
import java.util.List;

import javax.rad.model.SortDefinition;
import javax.rad.model.condition.Equals;
import javax.rad.server.SessionContext;
import javax.rad.type.bean.IBean;

import com.sibvisions.rad.persist.TriggerAPI;
import com.sibvisions.rad.persist.event.StorageEvent;
import com.sibvisions.rad.persist.jdbc.DBStorage;
import com.sibvisions.rad.server.security.AbstractSecurityManager;
import com.sibvisions.util.type.CommonUtil;

/**
 * The Users class is the life-cycle object for user management.
 *
 * @author Roland Hrmann
 */
public class Users extends Session
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Returns all Users.
	 * 
	 * @return all Users.
	 * @throws Exception 	if no USERS tables existing.
	 */
	public DBStorage getUsers() throws Exception
	{
		DBStorage dbs = (DBStorage)get("users");
		
		if (dbs == null)
		{
			dbs = new DBStorage();
			dbs.setDBAccess(getDBAccess());
			dbs.setWritebackTable("USERS");
			dbs.setDefaultSort(new SortDefinition("USERNAME"));
			dbs.open();
			dbs.eventBeforeInsert().addListener(this, "doUserCreated");
			dbs.eventBeforeUpdate().addListener(this, "doUserChanged");
						
			put("users", dbs);
		}
	
		return dbs;
	}
	
	/**
	 * Returns all user roles.
	 * 
	 * @return all user roles.
	 * @throws Exception 	if no USER_ROLE tables existing.
	 */
	public DBStorage getUserRoles() throws Exception
	{
		DBStorage dbs = (DBStorage)get("userRoles");
		
		if (dbs == null)
		{
			dbs = new DBStorage();
			dbs.setDBAccess(getDBAccess());
			dbs.setWritebackTable("USER_ROLE");
			dbs.open();
			dbs.eventBeforeInsert().addListener(this, "doCreated");
			dbs.eventBeforeUpdate().addListener(this, "doChanged");

			put("userRoles", dbs);
		}
	
		return dbs;
	}	
	
	/**
	 * Encrypts a password, if password is changed.
	 * 
	 * @param pEvent the storage event
	 * @throws Exception if encryption or data change fails
	 */
	private void encryptPwd(StorageEvent pEvent) throws Exception
	{
		IBean bn = pEvent.getNew();

		String sNew = (String)bn.get("PASSWORD");
		String sOld;

		IBean bnOld = pEvent.getOld();

		if (bnOld != null)
		{
			sOld = (String)bnOld.get("PASSWORD");
		}
		else
		{
			sOld = null;
		}

		if (!CommonUtil.equals(sOld, sNew))
		{
			bn.put("PASSWORD", AbstractSecurityManager.getEncryptedPassword(SessionContext.getCurrentSessionConfig(), sNew));
		}
	}
	
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * Sets the current User and date for CREATED_XXX + CHANGED_XXX.
	 * 
	 * @param pEvent the Storage event.
	 */
	public void doCreated(StorageEvent pEvent)
	{
		IBean newRow = pEvent.getNew();

		String sUser = TriggerAPI.getCurrentUserName();
		Timestamp timestamp = TriggerAPI.getCurrentTimestamp();
		
		newRow.put("CREATED_BY", sUser);
		newRow.put("CREATED_ON", timestamp);
		newRow.put("CHANGED_BY", sUser);
		newRow.put("CHANGED_ON", timestamp);
	}
	
	/**
	 * Sets the current User and date for CHANGED_XXX.	
	 * 
	 * @param pEvent the Storage event.
	 */
	public void doChanged(StorageEvent pEvent)
	{
		IBean newRow = pEvent.getNew();
		
		newRow.put("CHANGED_BY", TriggerAPI.getCurrentUserName());
		newRow.put("CHANGED_ON", TriggerAPI.getCurrentTimestamp());
	}	
	
	/**
	 * Sets the encrypted password.
	 * 
	 * @param pEvent the storage event
	 * @throws Exception if data change fails
	 */
	public void doUserChanged(StorageEvent pEvent) throws Exception
	{
		doChanged(pEvent);

		encryptPwd(pEvent);
	}

	/**
	 * Sets the encrypted password.
	 * 
	 * @param pEvent the storage event
	 * @throws Exception if data change fails
	 */
	public void doUserCreated(StorageEvent pEvent) throws Exception
	{
		doCreated(pEvent);
		
		encryptPwd(pEvent);
	}	
	
}	// Users
