/*
 * History
 *
 * 09.11.2011 - [RH] - creation
 * 06.04.2020 - [JR] - responsive support
 */
package com.sibvisions.module.user.screens;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import javax.rad.application.IWorkScreenApplication;
import javax.rad.application.genui.responsive.ResponsiveUtil.LayoutMode;
import javax.rad.genui.UIDimension;
import javax.rad.genui.UIImage;
import javax.rad.genui.UIInsets;
import javax.rad.genui.celleditor.UIImageViewer;
import javax.rad.genui.component.UIButton;
import javax.rad.genui.component.UILabel;
import javax.rad.genui.container.UIGroupPanel;
import javax.rad.genui.container.UIPanel;
import javax.rad.genui.control.UIEditor;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.genui.layout.UIFormLayout;
import javax.rad.io.IFileHandle;
import javax.rad.model.ColumnView;
import javax.rad.model.event.DataBookEvent;
import javax.rad.model.reference.ReferenceDefinition;
import javax.rad.model.ui.ITableControl;
import javax.rad.remote.AbstractConnection;
import javax.rad.ui.IAlignmentConstants;

import com.sibvisions.apps.components.AssignmentTable;
import com.sibvisions.apps.components.FilterEditor;
import com.sibvisions.apps.components.MorphPanel;
import com.sibvisions.apps.components.MorphPanel.DisplayMode;
import com.sibvisions.apps.components.MorphPanel.NavigationMode;
import com.sibvisions.apps.components.NavigationTable;
import com.sibvisions.apps.components.layout.AdvancedFormLayout;
import com.sibvisions.apps.projx.ProjXUtil;
import com.sibvisions.apps.projx.screens.DataSourceWorkScreen;
import com.sibvisions.rad.model.remote.RemoteDataBook;
import com.sibvisions.util.type.FileUtil;
import com.sibvisions.util.type.ImageUtil;

/**
 * The <code>UsersWorkScreen</code> manages application users.
 *
 * @author Roland Hrmann
 */
public class UsersWorkScreen extends DataSourceWorkScreen
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the default image when no profile image was found. */
	private static final String	NO_IMAGE			= "/com/sibvisions/module/user/images/nobody.gif";

	/** Users DataBook. */
	private RemoteDataBook		rdbUsers			= new RemoteDataBook();

	/** UserRole DataBook. */
	private RemoteDataBook		rdbUserRole			= new RemoteDataBook();

	/** Users Navigation table. */
	private NavigationTable		navUsers			= new NavigationTable();

	/** Roles Assignment table. */
	private AssignmentTable		atRoles				= new AssignmentTable();

	/** Editor for username. */
	private UIEditor			editUsername		= new UIEditor();

	/** Editor for active. */
	private UIEditor			editActive			= new UIEditor();

	/** Editor for password. */
	private UIEditor			editPassword		= new UIEditor();

	/** Editor for ChangePassword. */
	private UIEditor			editChangePassword	= new UIEditor();

	/** Editor for firstname. */
	private UIEditor			editFirstName		= new UIEditor();

	/** Editor for lastname. */
	private UIEditor			editLastName		= new UIEditor();

	/** Editor for email. */
	private UIEditor			editEmail			= new UIEditor();

	/** Editor for Phone. */
	private UIEditor			editPhone			= new UIEditor();

	/** Editor for Fax. */
	private UIEditor			editFax				= new UIEditor();

	/** Editor for profile image. */
	private UIEditor			editProfImage		= new UIEditor();

	/** The Border Layout for the container. */
	private UIBorderLayout		layoutThis			= new UIBorderLayout();

	/** Main Split Panel. */
	private MorphPanel			mpMain				= new MorphPanel();

	/** The Layout for users container. */
	private UIBorderLayout		layUsers			= new UIBorderLayout();

	/** The Panel for the users. */
	private UIPanel				panUsers			= new UIPanel();

	/** The profile image layout. */
	private UIFormLayout		layProfImage		= new UIFormLayout();

	/** The profile image panel. */
	private UIPanel				panProfImage		= new UIPanel();

	/** The add profile image button. */
	private UIButton			butSetProfImage		= new UIButton();

	/** The delete profile image button. */
	private UIButton			butDelProfImage		= new UIButton();

	/** The Layout for details. */
	private UIFormLayout		layDetails			= new UIFormLayout();

	/** The Panel for details. */
	private UIPanel				panDetails			= new UIPanel();

	/** The Layout for details. */
	private AdvancedFormLayout	layDetail			= new AdvancedFormLayout();

	/** The Panel for detail. */
	private UIGroupPanel		groupDetail			= new UIGroupPanel();

	/** The Layout for the roles. */
	private AdvancedFormLayout	layRoles			= new AdvancedFormLayout();

	/** The Panel for the roles. */
	private UIGroupPanel		groupRoles			= new UIGroupPanel();
	
	/** the user filter editor. */
	private FilterEditor 		filterSearch		= new FilterEditor();

	/** the search panel. */
	private UIPanel 			panSearch			= new UIPanel();
	
	/** the search layout. */
	private UIFormLayout 		laySearch			= new UIFormLayout();
	
	/** the search label. */
	private UILabel				labelSearch			= new UILabel();
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>UsersWorkScreen</code>.
	 * 
	 * @param pApplication 	the application.
	 * @param pConnection 	the connection
	 * @throws Exception 	if the init fails
	 */
	public UsersWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection) throws Exception
	{
		super(pApplication, pConnection);

		initializeModel();
		initializeUI();
	}

	/**
	 * Initialize model.
	 * 
	 * @throws Exception if the model couldn't initialized.
	 */
	private void initializeModel() throws Exception
	{
		UIImageViewer imageViewer = new UIImageViewer();
		imageViewer.setDefaultImageName(NO_IMAGE);
		imageViewer.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);

		rdbUsers.setDataSource(getDataSource());
		rdbUsers.setName("users");
		rdbUsers.open();

		rdbUsers.getRowDefinition().getColumnDefinition("ACTIVE").getDataType().setCellEditor(ProjXUtil.YESNO_EDITOR);
		rdbUsers.getRowDefinition().getColumnDefinition("PASSWORD").getDataType().setCellEditor(ProjXUtil.PASSWORD_EDITOR);
		rdbUsers.getRowDefinition().getColumnDefinition("CHANGE_PASSWORD").getDataType().setCellEditor(ProjXUtil.YESNO_EDITOR);

		rdbUsers.getRowDefinition().getColumnDefinition("PROFILE_IMAGE").getDataType().setCellEditor(imageViewer);

		rdbUsers.getRowDefinition().setColumnView(ITableControl.class, new ColumnView(new String[] { "USERNAME" }));

		rdbUserRole.setDataSource(getDataSource());
		rdbUserRole.setName("userRoles");
		rdbUserRole.setMasterReference(new ReferenceDefinition(new String[] { "USER_ID" }, rdbUsers, new String[] { "ID" }));
		rdbUserRole.open();

		rdbUserRole.eventBeforeDeleting().addListener(this, "doCheckAdminRole");
	}

	/**
	 * Initialize UI.
	 * 
	 * @throws Exception if the UI couldn't initialized.
	 */
	private void initializeUI() throws Exception
	{
		navUsers.setDataBook(rdbUsers);
		navUsers.setMaximumSize(new UIDimension(450, 350));

		layUsers.setMargins(5, 5, 5, 5);

		filterSearch.setDataRow(rdbUsers);
		filterSearch.setFullTextFilterColumnNames(new String[] {"FIRST_NAME", "LAST_NAME", "USERNAME", "EMAIL", "PHONE", "FAX"});
		
		labelSearch.setText("Search");
		
        laySearch.setMargins(2, 0, 10, 2);
        laySearch.setResponsive(false);
        
		panSearch.setLayout(laySearch);
		panSearch.add(labelSearch, laySearch.getConstraints(0, 0));
		panSearch.add(filterSearch, laySearch.getConstraints(1, 0, -1, 0));
		
		panUsers.setLayout(layUsers);

		panUsers.add(panSearch, UIBorderLayout.NORTH);
		panUsers.add(navUsers, UIBorderLayout.CENTER);

		editUsername.setDataRow(rdbUsers);
		editUsername.setColumnName("USERNAME");

		editActive.setDataRow(rdbUsers);
		editActive.setColumnName("ACTIVE");
		editActive.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		
		editPassword.setDataRow(rdbUsers);
		editPassword.setColumnName("PASSWORD");

		editChangePassword.setDataRow(rdbUsers);
		editChangePassword.setColumnName("CHANGE_PASSWORD");
		editChangePassword.setHorizontalAlignment(IAlignmentConstants.ALIGN_LEFT);
		
		editFirstName.setDataRow(rdbUsers);
		editFirstName.setColumnName("FIRST_NAME");

		editLastName.setDataRow(rdbUsers);
		editLastName.setColumnName("LAST_NAME");

		editEmail.setDataRow(rdbUsers);
		editEmail.setColumnName("EMAIL");

		editPhone.setDataRow(rdbUsers);
		editPhone.setColumnName("PHONE");

		editFax.setDataRow(rdbUsers);
		editFax.setColumnName("FAX");

		editProfImage.setDataRow(rdbUsers);
		editProfImage.setColumnName("PROFILE_IMAGE");
		editProfImage.setPreferredSize(140, 185);

		butSetProfImage.setBorderOnMouseEntered(true);
		butSetProfImage.setImage(UIImage.getImage(UIImage.ADD_SMALL));
		butSetProfImage.eventAction().addListener(this, "doChooseProfileImage");
		butSetProfImage.setFocusable(false);

		butDelProfImage.setBorderOnMouseEntered(true);
		butDelProfImage.setImage(UIImage.getImage(UIImage.REMOVE_SMALL));
		butDelProfImage.eventAction().addListener(this, "doDeleteProfileImage");
		butDelProfImage.setFocusable(false);

		atRoles.setDataBook(rdbUserRole);

        layProfImage.setHorizontalAlignment(IAlignmentConstants.ALIGN_CENTER);
        layProfImage.setMargins(0, 0, 0, 0);
        layProfImage.setVerticalGap(3);
        layProfImage.setResponsive(false);

		panProfImage.setLayout(layProfImage);
		panProfImage.add(editProfImage, layProfImage.getConstraints(0, 0, 1, 0));
		panProfImage.add(butSetProfImage, layProfImage.getLeftAlignedConstraints(0, 1));
		panProfImage.add(butDelProfImage, layProfImage.getRightAlignedConstraints(1, 1));

        layDetail.setAnchorConfiguration("l1=15,t3=15,t5=15");
        layDetail.setHorizontalLabelGap(5);
        layDetail.setVerticalLabelGap(3);
        layDetail.setLabelText(editChangePassword, "Password expired");
        layDetail.setLabelText(editFirstName, "First name");
        layDetail.setLabelText(editLastName, "Last name");

		groupDetail.setText("User details");
		groupDetail.setLayout(layDetail);
		groupDetail.add(panProfImage, layDetail.getConstraints(0, 0, 0, -1));
		groupDetail.add(editUsername, layDetail.getConstraints(1, 0));
		groupDetail.add(editActive, layDetail.getConstraints(-1, 0));
		groupDetail.add(editPassword, layDetail.getConstraints(1, 1));
		groupDetail.add(editChangePassword, layDetail.getConstraints(-1, 1));
		groupDetail.add(editEmail, layDetail.getConstraints(1, 2, -1, 2));
		groupDetail.add(editFirstName, layDetail.getConstraints(1, 3, -1, 3));
		groupDetail.add(editLastName, layDetail.getConstraints(1, 4, -1, 4));
		groupDetail.add(editPhone, layDetail.getConstraints(1, 5, 1, 5));
		groupDetail.add(editFax, layDetail.getConstraints(2, 5, -1, 5));
		
		groupRoles.setText("Roles");
		groupRoles.setLayout(layRoles);
		groupRoles.add(atRoles, layRoles.getConstraints(0, 0, -1, 0));

		layDetails.setVerticalAlignment(UIFormLayout.ALIGN_TOP);
		layDetails.setMargins(5, 5, 5, 5);

		panDetails.setLayout(layDetails);
		panDetails.add(groupDetail, layDetails.getConstraints(0, 0, -1, 0));
		panDetails.add(groupRoles, layDetails.getConstraints(0, 1, -1, 1));

		mpMain.setDisplayMode(DisplayMode.Split);
		mpMain.setInlineAndPopupNavigationMode(NavigationMode.SingleClick);
		mpMain.setButtonPanelMargins(new UIInsets(5, 5, 5, 5));
		mpMain.setNavigationPanelMargins(new UIInsets(5, 5, 5, 5));
		mpMain.setButtonPanelVisible(true);
		mpMain.setTabHeaderVisible(false);
		
		mpMain.add(panUsers, "User");
		mpMain.add(panDetails, "Detail");

		setLayout(layoutThis);
		add(mpMain, UIBorderLayout.CENTER);
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Overwritten methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void onResponsiveModeChanged(LayoutMode pOld, LayoutMode pNew) throws Throwable
	{
		super.onResponsiveModeChanged(pOld, pNew);

		if (pNew != null)
		{
			switch (pNew)
			{
				case Mini:
					rdbUsers.getRowDefinition().getColumnView(ITableControl.class).setColumnNames(new String[] {"USERNAME", "ACTIVE"});
					break;
				case Small:
					rdbUsers.getRowDefinition().getColumnView(ITableControl.class).setColumnNames(new String[] {"USERNAME", "ACTIVE", "EMAIL"});
					break;
				default:
					rdbUsers.getRowDefinition().getColumnView(ITableControl.class).setColumnNames(new String[] {"USERNAME"});
			}
		}
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// User-defined methods
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Sets the profile image for the current user.
	 * 
	 * @param pFileHandle the file.
	 * @throws Throwable if an error occurs.
	 */
	public void setProfileImage(IFileHandle pFileHandle) throws Throwable
	{
		String sFormat = FileUtil.getExtension(pFileHandle.getFileName().toLowerCase());

		if ("png".equals(sFormat) || "jpg".equals(sFormat) || "gif".equals(sFormat))
		{
			ByteArrayOutputStream stream = new ByteArrayOutputStream();

			ImageUtil.createScaledImage(pFileHandle.getInputStream(), 140, 185, true, stream, sFormat);

			stream.close();

			rdbUsers.setValue("PROFILE_IMAGE", stream.toByteArray());

			try
			{
				rdbUsers.saveSelectedRow();
			}
			catch (Exception pException)
			{
				// Silent Save of current row.
			}
		}
		else
		{
			throw new IOException("Image format '" + sFormat + "' not supported. Use 'png', 'jpg' or 'gif'!");
		}
	}

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Actions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Stores the relation between the Roles und Users.
	 * 
	 * @param pDataRowEvent 	the dataRow Event
	 * @throws Exception 		if a DB error occurs.
	 */
	public void doCheckAdminRole(DataBookEvent pEvent) throws Exception
	{
		String sUserName = getConnection().getUserName();

		if ("Administrator".equals(pEvent.getChangedDataBook().getValue("ROLE_NAME")) 
			&& sUserName != null && sUserName.equals(rdbUsers.getValue("USERNAME")))
		{
			throw new Exception("You can't remove the administrator role of yourself!");
		}
	}

	/**
	 * Starts the image upload.
	 * 
	 * @throws Throwable if an error occurs.
	 */
	public void doChooseProfileImage() throws Throwable
	{
		if (rdbUsers.getSelectedRow() >= 0)
		{
			getApplication().getLauncher().getFileHandle(this, "setProfileImage");
		}
	}

	/**
	 * Deletes the profile image.
	 * 
	 * @throws Throwable if an error occurs.
	 */
	public void doDeleteProfileImage() throws Throwable
	{
		rdbUsers.setValue("PROFILE_IMAGE", null);

		try
		{
			rdbUsers.saveSelectedRow();
		}
		catch (Exception pException)
		{
			// Silent Save of current row.
		}
	}

} 	// UsersWorkScreen
